# Generated by Django 4.2.13 on 2024-08-26 08:48

from django.db import migrations, models
from django.db.models import Exists, OuterRef
from enumfields import Enum, EnumField


class Scope(Enum):
    UploadS3Image = "upload_s3_image"
    CreateIIIFImage = "create_iiif_image"
    S3Ingest = "s3_ingest"
    CreateDockerWorkerVersion = "create_docker_worker_version"

def create_user_scope_booleans(apps, schema):
    UserScope = apps.get_model("users", "UserScope")
    User = apps.get_model("users", "User")

    User.objects.all().update(
        can_create_iiif_image=Exists(UserScope.objects.filter(user_id=OuterRef("pk"), scope=Scope.CreateIIIFImage)),
        can_create_worker_version=Exists(UserScope.objects.filter(user_id=OuterRef("pk"), scope=Scope.CreateDockerWorkerVersion)),
        can_ingest=Exists(UserScope.objects.filter(user_id=OuterRef("pk"), scope=Scope.S3Ingest)),
        can_upload_s3_image=Exists(UserScope.objects.filter(user_id=OuterRef("pk"), scope=Scope.UploadS3Image)),
    )

def create_user_scopes(apps, schema):
    UserScope = apps.get_model("users", "UserScope")
    User = apps.get_model("users", "User")

    scope_matching = [
        ("can_upload_s3_image", Scope.UploadS3Image),
        ("can_create_iiif_image", Scope.CreateIIIFImage),
        ("can_ingest", Scope.S3Ingest),
        ("can_create_worker_version", Scope.CreateDockerWorkerVersion)
    ]

    UserScope.objects.bulk_create(
        [
            UserScope(user_id=user.id, scope=scope)
            for user in User.objects.iterator()
            for scope_attr, scope in scope_matching
            if getattr(user, scope_attr)
        ]
    )

class Migration(migrations.Migration):

    dependencies = [
        ("users", "0003_delete_oauthcredentials"),
    ]

    operations = [
        migrations.AddField(
            model_name="user",
            name="can_create_iiif_image",
            field=models.BooleanField(default=False, help_text="Allows adding images to Arkindex from external IIIF servers using the CreateIIIFImage endpoint.\nThis could allow someone to make Arkindex perform DoS attacks on other IIIF servers due to the image checks."),
        ),
        migrations.AddField(
            model_name="user",
            name="can_create_worker_version",
            field=models.BooleanField(default=False, help_text="Allows creating workers and worker versions from Docker images."),
        ),
        migrations.AddField(
            model_name="user",
            name="can_ingest",
            field=models.BooleanField(default=False, help_text="Allows access to S3 ingestion features."),
        ),
        migrations.AddField(
            model_name="user",
            name="can_upload_s3_image",
            field=models.BooleanField(default=False, help_text="Allows uploading images to an S3 bucket using the CreateImage endpoint and `s3_put_url`.\nThis could allow someone to upload very large files or make DoS attacks on the local IIIF image server."),
        ),
        migrations.AlterField(
            model_name="userscope",
            name="scope",
            field=EnumField(enum=Scope, max_length=50),
        ),
        migrations.RunPython(
            create_user_scope_booleans,
            reverse_code=create_user_scopes,
            elidable=False
        ),
        migrations.DeleteModel(name="UserScope")
    ]
